/*
crstopov v 1.0 (December 2003)
Ian Scott (ian@norcimo.com)
*/

/* You may freely copy, redistribute and modify this code for non-commercial purposes provided you acknowledge the original author and make freely available, under identical conditions and restrictions, all code thus produced  */

/*

Converts VEGAS crystal files to POVRay views. The camera is initally placed centred above the crystal, looking straight down. Basic light sources are supplied at the centre of each face of a cubiod 20 units wider and centred on the crystal.

No bonds are added as of yet, a cylinder between atoms in the unitcell object is the obvious way. For small unit cells it shouldn't be too much of a problem to add by hand if desired.

Also it would be nice to relate sphere size to atomic radius in the future

Note that minimal error checking is performed in the code. The command line parameters are checked for basic errors, but the actual input crystal file is assumed to be correct. Use at your own risk.

*/


//Uncomment below for debug mode
#ifndef DEBUG
//#define DEBUG
#endif

#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#define MAXATOM 5000

int main(int argc, char* argv[])
{
#ifdef DEBUG
cout<<argc<<endl;
#endif
char *usagestring="crstopov <crystal file> [<Number x repeat> <Number y repeat>]";
//Check the number of command line arguments is correct

if (argc!=2 && argc!=4)
	{
	if (argc==1)
		cout<<"No crystal file specified!"<<endl;
	else if (argc==3)
		cout<<"Too few parameters (or maybe too many!)"<<endl;
	else
		cout<<endl<<"Too many parameters!"<<endl;
	cout<<usagestring<<endl<<endl;
	return 1;
	}


ifstream crsfile;
ofstream povfile;

float xcoord[MAXATOM], ycoord[MAXATOM], zcoord[MAXATOM], transx, transy;
char atype[MAXATOM][3], outname[255];
int i, nat,j, k, xcopy, ycopy;
char *dot;
//number of copies of unit cell
if (argc==2)
	xcopy=ycopy=1;
else
//check we've been supplied with something vaguely sensible
	{
	for (i=0;i<strlen(argv[2]);i++)
		{
		if (!isdigit(argv[2][i]))
			{
			cout<<"Non integer value!"<<endl;
			cout<<usagestring<<endl<<endl;
			return 4;
			}
		}
	for (i=0;i<strlen(argv[3]);i++)
		{
		if (!isdigit(argv[3][i]))
			{
			cout<<"Non integer value!"<<endl;
			cout<<usagestring<<endl<<endl;
			return 4;
			}
		}

	xcopy=atoi(argv[2]);
	ycopy=atoi(argv[3]);
	}
//open the input crystal file
crsfile.open(argv[1],ios::in);
if (crsfile==NULL)
	{
	cout<<"Couldn't open file "<<argv[1]<<endl;
	return 2;
	}

#ifdef DEBUG
cout<<"Input crs file opened!!"<<endl;
#endif
crsfile>>nat; //Number of atoms is the first thing in the crystal file
#ifdef DEBUG
cout<<"No. atoms: "<<nat<<endl;
#endif

//read in each atoms coords and the type
for (i=0;i<nat;i++)
	{
	crsfile>>xcoord[i];
	crsfile>>ycoord[i];
	crsfile>>zcoord[i];
	crsfile>>atype[i];
	atype[i][2]='\0';
#ifdef DEBUG
	cout<<"Input Crystal:"<<endl;
	cout<<"Atom No. "<<i+1<<": x, y, z, atom: "<<xcoord[i]<<", "<<ycoord[i]<<", "<<zcoord[i]<<", "<<atype[i]<<endl;
#endif
	}
crsfile>>transx;
crsfile>>transy;

crsfile.close();

//Setup output filename. This will be based on the input name but with pov as an extension
strcpy(outname,argv[1]);
dot=strrchr(outname,'.');
if (dot)
	strcpy(dot,".pov");
else
	strcat(outname,".pov");

#ifdef DEBUG
cout<<"Output filename: "<<outname<<endl;
#endif

//Open output file
#ifdef DEBUG
cout<<"xcopy: "<<xcopy<<" ycopy: "<<ycopy<<endl;
#endif
povfile.open(outname,ios::out);
#ifdef DEBUG
cout<<"Output POV file opened"<<endl;
#endif

if (povfile==NULL)
	{
cout<<"Couldn't open output file!"<<endl;
return 3;
	}

//Write POV scence, starting with basics
povfile<<"#include "<<'"'<<"colors.inc"<<'"'<<endl;
//define somee useful constants
povfile<<"#declare translatex="<<transx<<";"<<endl;
povfile<<"#declare translatez="<<transy<<";"<<endl;
povfile<<"#declare crystheight="<<-zcoord[nat-1]<<";"<<endl;
povfile<<"background {color Blue}"<<endl;
povfile<<"camera {"<<endl<<"location <translatex*"<<xcopy<<"/2, 20, translatez*"<<ycopy<<"/2>"<<endl<<"look_at <translatex*"<<xcopy<<"/2, 0, translatez*"<<ycopy<<"/2>"<<endl<<"}"<<endl<<endl;
//camera is centred above the crystal
povfile<<"light_source {<translatex*"<<xcopy<<"/2, 20, translatez*"<<ycopy<<"/2> White}"<<endl;
povfile<<"light_source {<translatex*"<<xcopy<<"/2, crystheight-20, translatez*"<<ycopy<<"/2> White}"<<endl;
povfile<<"light_source {<translatex*"<<xcopy<<"+20, crystheight/2, translatez*"<<ycopy<<"/2> White}"<<endl;
povfile<<"light_source {<-20, crystheight/2, translatez*"<<ycopy<<"/2> White}"<<endl;
povfile<<"light_source {<translatex*"<<xcopy<<"/2, crystheight/2, translatez*"<<ycopy<<"+20> White}"<<endl;
povfile<<"light_source {<translatex*"<<xcopy<<"/2, crystheight/2, -20> White}"<<endl;
//a cubiod 20 units from the crystal
povfile<<"#declare unitcell=union {"<<endl;
//our unit cell
for (i=0;i<nat;i++)
	{
	povfile<<"sphere { <"<<xcoord[i]<<", "<<-zcoord[i]<<", "<<ycoord[i]<<">, 0.5"<<endl;
//add in more atom types here. Default colour for unhandled types is Black
	if (strcmp("Si",atype[i])==0)
		{
		povfile<<"pigment {color Yellow}"<<endl;
		}
	if (strcmp("Ho",atype[i])==0)
		{
		povfile<<"pigment {color Grey}"<<endl;
		}

	povfile<<"}"<<endl;
	}
povfile<<"}"<<endl;
//Produce the appropriate number of copies of the unit cell
for (k=0;k<xcopy;k++)
	{
	for (j=0;j<ycopy;j++)
		{
		povfile<<"object {unitcell"<<endl;
		povfile<<"translate translatex*"<<k<<"*x"<<endl;
		povfile<<"translate translatez*"<<j<<"*z"<<endl;
		povfile<<"}"<<endl;
		}
	}
povfile.close();
cout<<"Done!! POVRay file is "<<outname<<endl;
}